@@ -8,7 +8,7 @@ use 5.008003;
 use File::Basename qw( basename );
 use List::Util     qw( none any );
 
-#use SQL::Type::Guess  qw(); # "require"d
+#use SQL::Type::Guess  qw(); # required
 
 use Term::Choose       qw( choose );
 use Term::Choose::Util qw( choose_a_number insert_sep );
@@ -17,9 +17,9 @@ use Term::TablePrint   qw( print_table );
 
 use App::DBBrowser::Auxil;
 use App::DBBrowser::DB;
+use App::DBBrowser::GetContent;
 use App::DBBrowser::Opt;
-use App::DBBrowser::Table;
-#use App::DBBrowser::Table::Insert; # "require"(d)
+use App::DBBrowser::Table::WriteAccess;
 
 
 sub new {
@@ -59,7 +59,7 @@ sub delete_table {
     if ( ! $ok ) {
         return;
     }
-    $ax->print_sql( $sql, [ $Drop_table ], undef, 'Computing: ... ' );
+    $ax->print_sql( $sql, [ $Drop_table ], 'Computing: ... ' );
     my $sth = $sf->{d}{dbh}->prepare( "SELECT * FROM " . $sql->{table} );
     $sth->execute();
     my $col_names = $sth->{NAME}; # mysql: $sth->{NAME} before fetchall_arrayref
@@ -90,6 +90,7 @@ sub delete_table {
 sub create_new_table {
     my ( $sf ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $gc = App::DBBrowser::GetContent->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $sql = {};
     $ax->reset_sql( $sql );
     my @cu_keys = ( qw/create_table_plain create_table_form_file create_table_form_copy settings/ );
@@ -133,18 +134,14 @@ sub create_new_table {
         }
         if ( $custom eq $cu{create_table_form_copy} ) {
             push @$stmt_typeS, 'Insert';
-            require App::DBBrowser::Table::Insert;
-            my $tbl_in = App::DBBrowser::Table::Insert->new( $sf->{i}, $sf->{o}, $sf->{d} );
-            my $ok = $tbl_in->from_copy_and_paste( $sql, $stmt_typeS );
+            my $ok = $gc->from_copy_and_paste( $sql, $stmt_typeS );
             if ( ! $ok ) {
                 next MENU;
             }
         }
         elsif ( $custom eq $cu{create_table_form_file} ) {
             push @$stmt_typeS, 'Insert';
-            require App::DBBrowser::Table::Insert;
-            my $tbl_in = App::DBBrowser::Table::Insert->new( $sf->{i}, $sf->{o}, $sf->{d} );
-            my $file_name = $tbl_in->from_file( $sql, $stmt_typeS );
+            my $file_name = $gc->from_file( $sql, $stmt_typeS );
             if ( ! $file_name ) {
                 next MENU;
             }
@@ -176,10 +173,8 @@ sub create_new_table {
         }
         $ax->print_sql( $sql, $stmt_typeS );
         if ( $custom eq $cu{create_table_plain} ) {
-            require App::DBBrowser::Table::Insert;
-            my $tbl_in = App::DBBrowser::Table::Insert->new( $sf->{i}, $sf->{o}, $sf->{d} );
             push @$stmt_typeS, 'Insert';
-            my $ok = $tbl_in->__from_col_by_col( $sql, $stmt_typeS );
+            my $ok = $gc->from_col_by_col( $sql, $stmt_typeS );
             if ( ! $ok ) {
                 pop @$stmt_typeS;
             }
@@ -212,8 +207,8 @@ sub create_new_table {
                 shift @columns;
             }
             $sql->{insert_into_cols} = $ax->quote_simple_many( \@columns );
-            my $tbl = App::DBBrowser::Table->new( $sf->{i}, $sf->{o}, $sf->{d} );
-            my $commit_ok = $tbl->commit_sql( $sql, $stmt_typeS );
+            my $tw = App::DBBrowser::Table::WriteAccess->new( $sf->{i}, $sf->{o}, $sf->{d} );
+            my $commit_ok = $tw->commit_sql( $sql, $stmt_typeS );
         }
         return 1;
     }
@@ -342,7 +337,7 @@ sub __set_data_types {
     my $data_types;
     my $fields;
     if ( $sf->{o}{create}{data_type_guessing} ) {
-        $ax->print_sql( $sql, $stmt_typeS, undef, 'Guessing data types ... ' );
+        $ax->print_sql( $sql, $stmt_typeS, 'Guessing data types ... ' );
         $data_types = $sf->__guess_data_type( $sql );
     }
     if ( defined $data_types ) {
